library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
 library UNISIM;
 use UNISIM.VComponents.all;

entity RAM_DAC is
    Port (Palette : in  STD_LOGIC_VECTOR (3 downto 0);  --Palette P[3:0] inputs. P4 is always high and P[7:6] are always low in the scope.
          Overlay : in  STD_LOGIC_VECTOR (3 downto 0);  -- Overlay O[3:0] inputs
		  Data : in STD_LOGIC_VECTOR (7 downto 0);  -- Data inputs D[7:0]
          RS : in  STD_LOGIC_VECTOR (2 downto 0);  --Register Select inputs R[2:0]
          WR_bar: in STD_LOGIC;  --/WR input
          clk : in  STD_LOGIC;  -- 25MHz clock input
		  blank_bar : in STD_LOGIC;  --/blank input
		  dataouta_p, dataouta_n	: out std_logic_vector(2 downto 0);	-- lvds data outputs
		  clkouta1_p, clkouta1_n	: out std_logic 			-- lvds clock output

 --          LCD_Out1, LCD_Out2, LCD_Out3 : out STD_LOGIC;  -- outputs to lvds converters to drive LCD inputs
 --         LCD_Clock_Out : out STD_LOGIC;  --  LVDS clock to LCD
 --         BL : in STD_LOGIC_VECTOR (2 downto 0);  --  inputs to set lcd backlight brightness (PWM).  From DIP switch on board. 000 = dimmest, 111 = brightest
 --       PWM : out STD_LOGIC;  -- PWM signal for LCD backlight.  Output to lcd to control backlight level
 --         LED_ON : out STD_LOGIC; -- LCD Backlight on/off control.  Turns on lcd backlight.  Always 1.
 --         --DEBUG
 --         led1 : buffer STD_LOGIC  --PLL locked indication
          --test_port_1 : out std_logic;
          --Debug_out : out std_logic_vector (3 downto 0)
	);
		
end RAM_DAC;

architecture Behavioral of RAM_DAC is
--  Components

	COMPONENT clock_generator
	PORT(
		CLKIN_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;
   
 signal clk_175 : std_logic;  --175MHz clock from clock generator.  Phase locked to input 25MHz pixel clock
 signal clk_25 : std_logic;  --25MHz clock from clock generator.  Phase locked to input 25MHz pixel clock.  Phase can be adjusted it required for timing if requied (using clock wizard)
 --signal clk_5 : std_logic;  --5MHz clock from clock generator.  Phase locked to input 25MHz pixel clock.  Used by backlight led pwm.
 signal locked : std_logic;  --locked fron clock generator
 signal register_mode : std_logic_vector (2 downto 0);  --Set by RS[2:0].  Latched on falling edge of WR_bar
 signal palette_address_register : std_logic_vector (7 downto 0);  --256 pallet RAM locations
 signal overlay_address_register : std_logic_vector (7 downto 0);  --15 overlay RAM locations
 type array_16x6 is array(15 downto 0) of std_logic_vector (5 downto 0);
 signal palette_RAM_red: array_16x6;  --only 16 locations are used in the scope.
 signal palette_RAM_green: array_16x6;
 signal palette_RAM_blue: array_16x6;
 signal overlay_RAM_red: array_16x6;  --There are only 15 overlay registers because location 0000 is reserved.
 signal overlay_RAM_green: array_16x6;
 signal overlay_RAM_blue: array_16x6;
 signal lvds_count: natural range 0 to 7; --used to count serial output clocks for the lcd lvds outputs
 signal synced: std_logic;  --used to sync the lvds_count with the start of the 25MHz clock
 signal de : std_logic;  --latched version of blank_bar.  Latched on rising edge of 25MHz clock
 signal red, grn, blu : std_logic_vector(5 downto 0);  --Latched version of the outputs of the palette and overlay RAM.  Latched on rising edge of 25MHz clock
 signal LCD_Out1, LCD_Out2, LCD_Out3 : STD_LOGIC;
 signal LCD_Clock_Out : STD_LOGIC;

begin
-- Connect Components
	inst_clock_generator: clock_generator PORT MAP(
		CLKIN_IN => clk,
		CLKFX_OUT => clk_175 ,
		CLK0_OUT => clk_25,
		LOCKED_OUT => locked
	);

 store_register_mode: process (WR_bar) -- store the register mode at the falling edge of WR_bar
 begin
    if falling_edge(WR_bar) then  --on falling edge of WR_bar, save the register mode and latch the address
        register_mode <= RS; 
     end if;
 end process store_register_mode;
     
 write_RAM: process (WR_bar)  --write the palette and overlay RAM on the rising edge of WR_bar.  
    variable address_a_b : natural range 0 to 2 := 0; --red/green/blue address counter bits(address_a address_b in data sheet)
 begin
    if rising_edge(WR_bar) then  --on rising edge of WR_bar, store the data in the address or RAM
            case (register_mode) is
                when "000" =>  -- Pallet RAM address register write (RAM write mode)
                    palette_address_register <= Data;
                    address_a_b := 0;
                when "100" =>  -- Overlay RAM address register write (RAM write mode)
                    overlay_address_register <= Data;
                    address_a_b := 0;
                when "001" =>  --Pallet RAM write
                    if address_a_b = 0 then
                        palette_RAM_red(to_integer(unsigned(palette_address_register(3 downto 0)))) <= Data(5 downto 0); --write to red pallet RAM
                    elsif address_a_b = 1 then
                        palette_RAM_green(to_integer(unsigned(palette_address_register(3 downto 0)))) <= Data(5 downto 0); --write to green pallet RAM
                    elsif address_a_b = 2 then
                        palette_RAM_blue(to_integer(unsigned(palette_address_register(3 downto 0)))) <= Data(5 downto 0); --write to blue pallet RAM
                    end if;
                    if address_a_b < 2 then
                        address_a_b := address_a_b + 1;
                    else
                        address_a_b := 0;
                        palette_address_register <= std_logic_vector(unsigned(palette_address_register) + 1);
                    end if;
                when "101" =>  --Overlay RAM write
                    if address_a_b = 0 then
                        overlay_RAM_red(to_integer(unsigned(overlay_address_register(3 downto 0)))) <= Data(5 downto 0); --write to red pallet RAM
                    elsif address_a_b = 1 then
                        overlay_RAM_green(to_integer(unsigned(overlay_address_register(3 downto 0)))) <= Data(5 downto 0); --write to green pallet RAM
                    elsif address_a_b = 2 then
                        overlay_RAM_blue(to_integer(unsigned(overlay_address_register(3 downto 0)))) <= Data(5 downto 0); --write to blue pallet RAM
                    end if;
                    if address_a_b < 2 then
                        address_a_b := address_a_b + 1;
                    else
                        address_a_b := 0;
                        overlay_address_register <= std_logic_vector(unsigned(overlay_address_register) + 1);
                    end if;
                when others =>  --other functions (pixel mask, register reads) are not used by the scope.
                    address_a_b := 0;
            end case;            
  end if;
 end process write_RAM; 
     


 process (CLK_175, CLK_25)  
 variable lvds_count: natural range 0 to 7:= 0;
 variable synced: bit := '0';
	  begin
			if rising_edge(CLK_175) then
				 if CLK_25 = '1' and synced = '0' then
					  synced := '1';					  
					  de <= blank_bar;
						 case (Overlay) is
							  when "0000" =>
									red <= palette_RAM_red(to_integer(unsigned(palette)));
									grn <= palette_RAM_green(to_integer(unsigned(palette)));
									blu <= palette_RAM_blue(to_integer(unsigned(palette)));
							  when others =>
									red <= overlay_RAM_red(to_integer(unsigned(overlay)));
									grn <= overlay_RAM_green(to_integer(unsigned(overlay)));
									blu <= overlay_RAM_blue(to_integer(unsigned(overlay)));
						 end case;
					  lvds_count := 0; 
				 else
					  if lvds_count < 6 then
							lvds_count := lvds_count +1;  
					  end if;
				 end if;
				 
				 case (lvds_count) is
				 when 0 =>
					  LCD_Out1 <= grn(0);
					  LCD_Out2 <= blu(1);
					  LCD_Out3 <= de;
					  LCD_Clock_Out <= '1';
				when 1 =>
					  LCD_Out1 <= red(5);
					  LCD_Out2 <= blu(0);
					  LCD_Out3 <= '0';
					  LCD_Clock_Out <= '1';
			  when 2 =>
					  LCD_Out1 <= red(4);
					  LCD_Out2 <= grn(5);
					  LCD_Out3 <= '0';
					  LCD_Clock_Out <= '0';
			  when 3 =>
					  LCD_Out1 <= red(3);
					  LCD_Out2 <= grn(4);
					  LCD_Out3 <= blu(5);
					  LCD_Clock_Out <= '0';
			  when 4 =>
					  LCD_Out1 <= red(2);
					  LCD_Out2 <= grn(3);
					  LCD_Out3 <= blu(4);
					  LCD_Clock_Out <= '0';
			  when 5 =>
					  LCD_Out1 <= red(1);
					  LCD_Out2 <= grn(2);
					  LCD_Out3 <= blu(3);
					  LCD_Clock_Out <= '1';
					  synced := '0';
			  when 6 =>
					  LCD_Out1 <= red(0);
					  LCD_Out2 <= grn(1);
					  LCD_Out3 <= blu(2);
					  LCD_Clock_Out <= '1';
				 when others =>   
					  LCD_Out1 <= red(0);
					  LCD_Out2 <= grn(1);
					  LCD_Out3 <= blu(2);
					  LCD_Clock_Out <= '1';
			end case;
			
	  end if;
 end process;

lvds_clka_obuf  : obufds generic map (IOSTANDARD => "LVDS_33") 
	port map(i => LCD_Clock_Out, 	o => clkouta1_p,  	oB => clkouta1_n );

lvds_0_obuf  : obufds generic map (IOSTANDARD => "LVDS_33") 
	port map(i => LCD_Out1, 	o => dataouta_p(0),  	oB => dataouta_n(0) );

lvds_1_obuf  : obufds generic map (IOSTANDARD => "LVDS_33") 
	port map(i => LCD_Out2, 	o => dataouta_p(1),  	oB => dataouta_n(1) );

lvds_2_obuf  : obufds generic map (IOSTANDARD => "LVDS_33") 
	port map(i => LCD_Out3, 	o => dataouta_p(2),  	oB => dataouta_n(2) );

	
end Behavioral;



